// Based on sample C code from R W Grout.
//
// This plugin reads a python script file from the input directory
// which prepares a VTK pipeline for execution each time the
// simulation code completes some processing.

#include "vtkMPIController.h"
#include "vtkInsituPVBatch.h"
#include "vtkDotSource.h"
#include "vtkStdString.h"

#include <vtkstd/vector>

#include <vtksys/SystemTools.hxx>

#include <stdio.h>
#include <unistd.h>

vtkMultiProcessController* pvDotController = 0;
vtkDotSource* pvDotSource = 0;
vtkInsituPVBatch* pvDotBatch = 0;
int pvDotCallCount = 0;
double pvDotPrevTime = 0.;

void pvdotpython_read_script( vtkStdString& initScript )
{
  initScript.clear();

  char line[256]; // FIXME
  ifstream is;
  is.open( "insitu.py", ifstream::in );
  while ( is.good() )
    {
    is.getline( line, 256 );
    initScript += line;
    initScript += "\n";
    }
  is.close();
}

extern "C" {

static int nameSize = 25;

void pvdotpython_init( int local_id, int gcomm, int num_particles, double* pxyz, double* mass, double* pmom, double* pfrc )
{
  if ( ! local_id )
    {
    printf("Running pvdotpython_init\n" );
    }

  pvDotCallCount = 0;
  pvDotPrevTime = 0.;
  vtkStdString initScript;
  pvdotpython_read_script( initScript );

  pvDotBatch = vtkInsituPVBatch::New();
  pvDotBatch->Initialize( initScript );

  pvDotController = vtkMultiProcessController::GetGlobalController ();
  pvDotSource = vtkDotSource::New();
  pvDotSource->Initialize( num_particles, pxyz, mass, pmom, pfrc, local_id, pvDotController->GetNumberOfProcesses() );

  // pvDotBatch->SetDataSource( 0, pvDotSource );
}

void pvdotpython_step( int myid, int gcomm, double time )
{
  if ( ! myid )
    {
    printf("Running pvdotpython_step at time %e on process %d\n", time, myid);
    }

  if ( pvDotSource->IsInitialized() )
    {
    pvDotSource->UpdateSimulationTime( time );
    }
  else
    {
    vtkGenericWarningMacro( << "Dot source not properly initialized. Dimensional sim time = " << time );
    return;
    }

  double dt = time - pvDotPrevTime;
  pvDotBatch->Update( pvDotCallCount++, time, dt );
  return;
}

void pvdotpython_fini( int myid, int gcomm )
{
  if ( ! myid )
    {
    printf("Running pvdotpython_fini\n" );
    }
  pvDotSource->Delete();

  pvDotBatch->Finalize();
  pvDotBatch->Delete();

  vtkDotSource::DestroySingleton ();
}


} // namespace "C"

